library ieee;
use ieee.std_logic_1164.all;

architecture collections of test_bench is

    constant stimulus_vector_length : natural := 10;

    type stimulus_element is
	record
            application_time : delay_length;
            pattern : std_logic_vector (0 to stimulus_vector_length - 1);
	end record stimulus_element;
    
    function stimulus_key ( stimulus : stimulus_element ) return delay_length is
    begin
	return stimulus.application_time;
    end function stimulus_key;
    
    function less_than ( L, R : delay_length ) return boolean is
    begin
        return L < R;
    end function less_than;

    package ordered_stimulus_collection_adt is
	new ordered_collection_adt
--            generic map ( element_type => stimulus_element,
--                          key_type => delay_length,
--                          key_of => stimulus_key,
--                          "<" => std.standard."<" );
            generic map ( stimulus_element,
                          delay_length,
                          stimulus_key,
                          less_than );
        
    signal dut_inputs : std_logic_vector (0 to stimulus_vector_length - 1);

begin

    stimulus_generator : process is

        use ordered_stimulus_collection_adt.all;
                                 
        variable dut_stimuli : ordered_collection := new_ordered_collection;
                                 
        procedure apply_stimulus ( stimulus : stimulus_element ) is
        begin
            wait for stimulus.application_time - now;
            dut_inputs <= stimulus.pattern;
        end procedure apply_stimulus;
                                 
        procedure apply_all_stimuli is new traverse
            generic map ( action => apply_stimulus );
                                       
    begin
        apply_all_stimuli ( dut_stimuli );
        wait;
    end process stimulus_generator;

end architecture collections;
